# JW120619: Code sent from David Doonan, GPUS (gp.org)
#
Methods:
# $sess_ref = WebDB::Session->open ($dbh, $sess_id)
#	$dbh should be existing database handle, or undef to use default
#   $sess_id should be an existing ID or undef; return reference to session
#   object.  The reference can be used like this:
#    $sess_ref->attr ($name, $value);	# assign value to session attribute
#    $value = $sess_ref->attr ($name);	# access session attribute
# $sess_ref = WebDB::Session->open_with_expiration($dbh, $sess_id)
#    Like open() but deletes the session if it has expired and returns undef.
# $sess_ref->close ()
#   close session
# $sess_ref->delete ()
#   delete (destroy) session

#@ PREAMBLE
package WebDB::Session;

use strict;
use DBI;
use Apache::Session::MySQL;

# global variable for error message

$WebDB::Session::errstr = "";
#@ PREAMBLE

# Open a session

#@ OPEN_WITHOUT_EXPIRE
sub open
{
my ($type, $dbh, $sess_id) = @_;
my %session;	# session hash
my %attr;		# connection attributes
# default connection parameters
my $dsn = "DBI:mysql:host=localhost;database=webdb";
my $user_name = "webdev";
my $password = "webdevpass";

	# Set up connection attributes according to whether
	# or not we're using an existing connection

	if (defined ($dbh))		# use existing connection
	{
		%attr =
		(
			Handle => $dbh,
			LockHandle => $dbh
		);
	}
	else					# use default connection parameters
	{
		%attr =
		(
			DataSource => $dsn,
			UserName => $user_name,
			Password => $password,
			LockDataSource => $dsn,
			LockUserName => $user_name,
			LockPassword => $password
		);
	}
	eval			# open session, putting tie within eval to trap errors
	{
		tie %session, "Apache::Session::MySQL", $sess_id, \%attr;
	};
	if ($@)			# session establishment failed
	{
		$WebDB::Session::errstr = $@;	# save error message
		return undef;
	}

	# get reference to session hash, bless it into class, and return it
	return (bless (\%session, $type));
}
#@ OPEN_WITHOUT_EXPIRE

# Open a session, but check the expiration date.  If the session has
# expired, delete it and return undef.

#@ OPEN_WITH_EXPIRATION
sub open_with_expiration
{
my $self = &open (@_);

	# If session exists and has an expiration time that
	# has passed, clobber it.

	if (defined ($self) && defined ($self->expires ())
		&& $self->expires () < $self->now ())
	{
		$WebDB::Session::errstr = sprintf ("Session %s has expired",
												$self->session_id ());
		$self->delete ();
		$self = undef;
	}
	return ($self);
}
#@ OPEN_WITH_EXPIRATION


# attr() - get or set value of session attribute:
# $sess_ref->attr ("attribute name");			# return value of attribute
# $sess_ref->attr ("attribute name", $value);	# set value of attribute
												# (also returns new value)

#@ ATTR
sub attr
{
my $self = shift;

	return (undef) unless @_;			# no arguments
	$self->{$_[0]} = $_[1] if @_ > 1;	# two arguments; set value first
	return ($self->{$_[0]});			# return current value
}
#@ ATTR

#@ SESSION_ID
sub session_id
{
my $self = shift;

	return ($self->{_session_id});
}
#@ SESSION_ID

# Call with no argument to retrieve expiration time, with 1 argument to
# set the value.

sub expires_1
#@ EXPIRES_1
{
my $self = shift;

	return ($self->attr ("#<expires>#", @_));
}
#@ EXPIRES_1

#@ EXPIRES_2
use Time::Local;	# this module is needed because it contains timegm()

sub expires
{
my $self = shift;
my $expires;

	# If an argument was given, interpret it as a local time() value in
	# seconds; convert the value to an absolute GMT date array and store
	# a reference to it.
	$self->{"#<expires>#"} = [ gmtime ($_[0]) ] if @_;

	# Return current expiration value (if there is one) by converting the
	# GMT date array back to a local time() value in seconds.  Return undef
	# if there is no expiration.
	$expires = $self->{"#<expires>#"};
	$expires = timegm (@{$expires}) if defined ($expires);
	return ($expires);
}
#@ EXPIRES_2

# Return current time is same units that are used to represent expiration
# values

#@ NOW
sub now
{
	return (time ());
}
#@ NOW

# Close or delete a session

#@ CLOSE_DELETE
sub close
{
my $self = shift;

	untie (%{$self});
}

sub delete
{
my $self = shift;

	tied (%{$self})->delete ();
}
#@ CLOSE_DELETE

1;
