#!/usr/bin/perl -w

#JGW120627: Added the 'irvtype' field to the irv table create statement.
#  The adminirv script needs to test and set the value or it crashes.
#
#Copyright (C) 2004 Susan J. Dridi sdridi@greens.org
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#IMPORTANT:
#Please read the comments at the top and bottom of this program - you must fill in the requested info or the program will not work.
#You need to enter the path to the Perl modules that came with votingpages.
#You also need to enter the name of your database and the username and password for the user that has CREATE and DROP TABLE privileges.
#You will need to enter a votingpages user with admin priviledges at the bottom of this script.
#The program will not work unless you make these modifications!

use strict;
use DBI;
use DBI;
use strict;
use warnings;
#Put the path to the Perl modules that came with votingpages in the use lib statement below. Don't include a trailing slash.
# jgw120717: The setup.pl script changes LIB to the path to the Perl modules the admin enters when prompted to do so. 
#   It does the same thing the rest of the modules and to all the scripts with a qw(LIB) statement.
#
use lib qw(LIB);
use Conf;

#Create tables
#Run this once when you are setting  up.

my $dbh = DBI->connect (
	'DBI:mysql:gpca_votingdb',	#add your database name after the 2nd colon
	'xxxxxxxx',		#add the user name with all priviledges in the quotes.
	'xxxxxxxx'		#add the password for that user in the quotes.
	);

$dbh->do('DROP TABLE IF EXISTS greenid');
$dbh->do('DROP TABLE IF EXISTS propid');
$dbh->do('DROP TABLE IF EXISTS candid');
$dbh->do('DROP TABLE IF EXISTS signons');
$dbh->do('DROP TABLE IF EXISTS greens');
$dbh->do('DROP TABLE IF EXISTS roles');
$dbh->do('DROP TABLE IF EXISTS greens_roles');
$dbh->do('DROP TABLE IF EXISTS greens_bios');
$dbh->do('DROP TABLE IF EXISTS districts');
$dbh->do('DROP TABLE IF EXISTS disttypes');
$dbh->do('DROP TABLE IF EXISTS proposals');
$dbh->do('DROP TABLE IF EXISTS phases');
$dbh->do('DROP TABLE IF EXISTS votes');
$dbh->do('DROP TABLE IF EXISTS irv');
$dbh->do('DROP TABLE IF EXISTS irvvotes');
$dbh->do('DROP TABLE IF EXISTS cands');
$dbh->do('DROP TABLE IF EXISTS committees');
$dbh->do('DROP TABLE IF EXISTS states');
# $dbh->do('DROP TABLE IF EXISTS counties');

#####################################
# jgw120617: All of the create statments below caused an SQL Syntax error at 'TYPE=InnoDB' on MySQL 5.3.
#   I changed 'TYPE' to 'ENGINE' and MySQL accepted them. 
#
#   The greenid, propid and candid table values being set below could later be kept in, retrieved from and updated in a 
#   a vsystem_variables table of key-value pairs of which just named three would be one of a possible many pairs.  
#   I also recommend there later be two new tables: "repunitid" and "repunits."  With them more than one possible repunit be  
#   defined as individual records in the repunits table instead of just the one pre-set by 'admin' at pre-install time in the 
#   Conf.pm module and 'sub repunit' value-retrieval subroutine therein.  Then the next 'admin' definition of a new 'repunit' 
#   key-value pair goes into the vsystem_variables table and the repunitid variable value gets bumped to the next number, n+1, 
3   where 'n' is the total number of existing repunit key-value pairs in the repunits table. 
#
#####################################

# jgw120717: The greenid table is a one record table that contains the next green id (gid in the greens and other tables)  
#   which the Voting System will use when the 'admin' user tries to enter a new green into the greens table. 
#
$dbh->do('CREATE TABLE greenid (greenid int unsigned NOT NULL, PRIMARY KEY(greenid)) MAX_ROWS = 1 ENGINE=InnoDB');

# jgw120717: The propid table is a one record table that contains the next yes/no proposal id or irv proposal id 
#   which the voting system will use when the 'admin' user tries to enter a new yes/no or irv proposal.
#
$dbh->do('CREATE TABLE propid (propid int unsigned NOT NULL, PRIMARY KEY(propid)) MAX_ROWS = 1 ENGINE=InnoDB');

# jgw120717: The candid table is a one record table that contains the next id number (cid) for the next irv or ranked choice 
#   candidate or choice the voting system will use when the 'admin' user data enters either kind of new proposal.
#   Entering an irv proposal requires the 'admin' to enter at least one candidate (or choice) for the irv before the irv proposal
#   is accepted and stored in the irv table; the one candidate gets stored in the cands table.  Thereafter the irv proposal reappears
#   in the adminirv script's editing web page (form) allowing the 'admin' user to correct the proposal descrition, number of winning
#   seats, etc. but also to add additional candidates (or choices), delete some of them, or edit their names.
#  
$dbh->do('CREATE TABLE candid (candid int unsigned NOT NULL, PRIMARY KEY(candid)) MAX_ROWS = 1 ENGINE=InnoDB');

# jgw120717: A signons table record will be created whenever the 'admin' user creates a new green and that new green is to be a 
#   voting delegate or alternate voter (aka proxy) in a delegation.  Thereafter that green can log in and edit his/her password  
#   a single delegates info web page (admindel) which the system takes them to only when the delegate clicks on his/her name in 
#   a list of delegates.  This list is produced by the admindelegates script which non-admin logged in system users and ordinary 
#   Voting System (aka votingpages) site visitors can get to from the contacts script's page.  The contacts script's web page
#   displays a "See all delegates" link mid-page. 
#
$dbh->do('CREATE TABLE signons (gid int unsigned NOT NULL, signon varchar(20) NOT NULL, pw varchar(15) NOT NULL, lastlogin datetime, sentreminder tinyint unsigned, PRIMARY KEY(gid), UNIQUE KEY signon(signon), KEY sentreminder(sentreminder)) ENGINE=InnoDB');

# jgw120617: Got a SQL Syntax error at 'TYPE=InnoDB' so I changed 'TYPE' to 'ENGINE'
$dbh->do('CREATE TABLE greens (gid int unsigned NOT NULL, fname varchar(20) NOT NULL, lname varchar(20) NOT NULL, add1 varchar(25), add2 varchar(20), city varchar(25), stateabbr varchar(2), zip varchar(10), email varchar(80), phd varchar(20), phe varchar(20), phc varchar(20), phf varchar(20), moddate timestamp(8) NOT NULL, PRIMARY KEY(gid), KEY lname (lname), KEY fname (fname), KEY moddate(moddate)) ENGINE=InnoDB');

$dbh->do('CREATE TABLE roles (roleid smallint unsigned NOT NULL auto_increment, role varchar(25) NOT NULL, PRIMARY KEY(roleid)) ENGINE=InnoDB');

# jgw120617: Got a SQL Syntax error at 'TYPE=InnoDB' so I changed 'TYPE' to 'ENGINE'
$dbh->do('CREATE TABLE greens_roles (gid int unsigned NOT NULL, commid smallint unsigned NOT NULL, roleid smallint unsigned NOT NULL, distid smallint unsigned NOT NULL, actstatus tinyint unsigned NOT NULL default \'1\', moddate timestamp(8) NOT NULL, PRIMARY KEY(gid,commid,roleid,distid), KEY actstatus (actstatus), KEY moddate(moddate)) ENGINE=InnoDB');

$dbh->do('CREATE TABLE greens_bios (gid int unsigned NOT NULL, bio text NOT NULL, pic tinyint unsigned NOT NULL, PRIMARY KEY(gid), KEY pic(pic)) ENGINE=InnoDB');

# jgw120617: Got a SQL Syntax error at 'TYPE=InnoDB' so I changed 'TYPE' to 'ENGINE'
$dbh->do('CREATE TABLE districts (distid smallint unsigned NOT NULL auto_increment, abbr varchar(5), district varchar(30) NOT NULL, disttypeid tinyint unsigned NOT NULL, voters smallint unsigned NOT NULL, maxvoters smallint unsigned NOT NULL, accreddate date, website varchar(100), actstatus tinyint unsigned NOT NULL default \'1\', moddate timestamp(8) NOT NULL, PRIMARY KEY(distid), UNIQUE KEY district(district), KEY disttypeid (disttypeid), KEY voters (voters), KEY maxvoters (maxvoters), KEY accreddate(accreddate), KEY actstatus (actstatus), KEY moddate (moddate)) ENGINE=InnoDB');

$dbh->do('CREATE TABLE disttypes (disttypeid smallint unsigned NOT NULL auto_increment, disttype varchar(10) NOT NULL, PRIMARY KEY(disttypeid), KEY disttype (disttype)) ENGINE=InnoDB');

# jgw120617: Got a SQL Syntax error at 'TYPE=InnoDB' so I changed 'TYPE' to 'ENGINE'
$dbh->do('CREATE TABLE committees (commid smallint unsigned NOT NULL auto_increment, comm varchar(50) NOT NULL, webpage varchar(100), listserv varchar(100), actstatus tinyint unsigned NOT NULL default \'1\', moddate timestamp(8) NOT NULL, PRIMARY KEY(commid), KEY actstatus(actstatus), KEY moddate(moddate)) ENGINE=InnoDB');

$dbh->do('CREATE TABLE states (abbr varchar(2) NOT NULL, state varchar(30) NOT NULL, PRIMARY KEY(abbr)) ENGINE=InnoDB');

# jgw120616: I added & edited a clone of the 'states' table, which be default at install time has all the states inserted into it,
#   which cloned-table I renamed the 'counties' table. The former 'abbr' field in the 
#   new 'counties' table is now varchar (3) (changed from 2 to 3) and contains a three alpha caps abbreviation for each California county.
#   The normal abbr field in the 'state' table, a field which is used in the code, I added to the 'counties' table as a new 1st field in 
#   the counties table.  I call the new 1st field in the 'counties' table: 'stabbr'.  It is not used in any of the Voting System code.   
#   The 'counties' table is not used for anything yet in the code either.  But it has real California county data in it which I added manually. 
# jgw120717: The county abbreviations which I added manually in the greens table records that have real greens data
#   also are not being used. The code to use the values is not yet un-commented in the edited scripts that should perform such actions
#   if/when the Voting System is enabled to serve smaller grained voting groups within a state and smaller grained inter-state groups
#   that include greens in multiple counties within several states.
#
#$dbh->do('CREATE TABLE counties (stabbr varchar (2) abbr varchar(3) NOT NULL, county varchar(30) NOT NULL, PRIMARY KEY(abbr)) ENGINE=InnoDB');

$dbh->do('CREATE TABLE proposals (pid smallint unsigned NOT NULL, title varchar(100) NOT NULL, gid int unsigned NOT NULL, presenter varchar(100) NOT NULL, background text NOT NULL, proposal text NOT NULL, resources text, reference text, discussbegin date NOT NULL, discussend date NOT NULL, votebegin date NOT NULL, voteend date NOT NULL, presens double unsigned NOT NULL, presensval smallint unsigned, consens double unsigned NOT NULL, consensval smallint unsigned, phaseid tinyint unsigned NOT NULL, distnum smallint unsigned, result varchar(30), moddate timestamp NOT NULL, PRIMARY KEY(pid), KEY gid(gid), KEY discussbegin (discussbegin), KEY discussend (discussend), KEY votebegin (votebegin), KEY voteend (voteend), KEY presens (presens), KEY consens (consens), KEY phaseid (phaseid), key result (result), key moddate (moddate)) ENGINE=InnoDB');

$dbh->do('CREATE TABLE phases (phaseid tinyint unsigned NOT NULL auto_increment, phase varchar(25) NOT NULL, PRIMARY KEY(phaseid)) ENGINE=InnoDB');

$dbh->do('CREATE TABLE votes (pid smallint unsigned NOT NULL, gid int unsigned NOT NULL, distid smallint unsigned NOT NULL, roleid smallint unsigned NOT NULL, vote varchar(10) NOT NULL, moddate timestamp NOT NULL, PRIMARY KEY(pid,gid,distid), KEY roleid(roleid), KEY vote(vote), KEY moddate(moddate)) ENGINE=InnoDB');

# jgw120717: I added the 'resources' and 'references' fields to the copy of the create irv statement as Mike Feinstein requested in late June. 
#   But 'resources' and 'references' field values are not enabled to be either displayed or editable in the adminirv script.  
#   See adminirv.v3 for commented out code that still doesn't work but which I .
#   The length of the title varchar field is extended in the copy of the create irv statement as MF requested in early July from 100 to 150.
#   Extending to 255 may be more suitable if/when proposal titles become twice as long. The 150 varchar title field seems to display aok. 
#     Feinstain also wants there to be a consens (and thus also a calculated Consens Quorum value) for an IRV / RCV proposal.  I have not 
#   added those fields to the irv table or added commented-out code taken from the adminprop script to allow the 'admin' user to edit those
#   new values.  But those steps I intended to perform once the Voting Systems as-it-is works "normally," i.e. as normal as has 
#   the GP-US version in the last so-many years.
#
#$dbh->do('CREATE TABLE irv (pid smallint unsigned NOT NULL, title varchar(100) NOT NULL, gid int unsigned NOT NULL, numseats tinyint unsigned NOT NULL, irvtype tinyint unsigned NOT NULL, background text, discussbegin date NOT NULL, discussend date NOT NULL, votebegin date NOT NULL, voteend date NOT NULL, presens double unsigned NOT NULL, presensval smallint unsigned, phaseid tinyint unsigned NOT NULL, distnum smallint unsigned, moddate timestamp NOT NULL, PRIMARY KEY(pid), KEY gid (gid), KEY numseats(numseats), KEY discussbegin (discussbegin), KEY discussend (discussend), KEY votebegin (votebegin), KEY voteend (voteend), KEY phaseid (phaseid), key moddate (moddate)) ENGINE=InnoDB');
$dbh->do('CREATE TABLE irv (pid smallint unsigned NOT NULL, title varchar(150) NOT NULL, gid int unsigned NOT NULL, numseats tinyint unsigned NOT NULL, irvtype tinyint unsigned NOT NULL, background text, resources text, references text, discussbegin date NOT NULL, discussend date NOT NULL, votebegin date NOT NULL, voteend date NOT NULL, presens double unsigned NOT NULL, presensval smallint unsigned, phaseid tinyint unsigned NOT NULL, distnum smallint unsigned, moddate timestamp NOT NULL, PRIMARY KEY(pid), KEY gid (gid), KEY numseats(numseats), KEY discussbegin (discussbegin), KEY discussend (discussend), KEY votebegin (votebegin), KEY voteend (voteend), KEY phaseid (phaseid), key moddate (moddate)) ENGINE=InnoDB');

# jw120617: SQL Syntax error at 'TYPE=InnoDB' so I changed 'TYPE' to 'ENGINE'
$dbh->do('CREATE TABLE cands (candid smallint unsigned NOT NULL, pid smallint unsigned NOT NULL, cand varchar(50) NOT NULL, info text, resources text, references text, candtype tinyint unsigned NOT NULL, equiv tinyint unsigned, result varchar(30), actstatus tinyint unsigned NOT NULL default \'1\', moddate timestamp NOT NULL, PRIMARY KEY(candid,pid), KEY cand (cand), KEY candtype(candtype), KEY equiv(equiv), KEY result(result), KEY actstatus(actstatus), KEY moddate(moddate)) ENGINE=InnoDB');

$dbh->do('CREATE TABLE irvvotes (pid smallint unsigned NOT NULL, voterkey varchar(60) NOT NULL, candid smallint unsigned NOT NULL, candrank tinyint unsigned NOT NULL, PRIMARY KEY(pid,voterkey,candid), KEY candrank(candrank)) ENGINE=InnoDB');

#Add values for supporting tables.
#  jgw120717: To create initial conditions that allow data entry of real greens info and real proposal info, 
#    and real committees and districts as well.
#  A new installer must have already edited the Conf.pm module to supply a value for the $group string. See Dridi's comments in the 
#  Conf.pm module for instructions on what to enter as a suitable string for your installation. 
#  
# The group() subroutine called just below retrieves the pre-set value for the $group variable stored in the Conf.pm module, an 
# apparent Dridi hack in 2004-06 to avoid using global constants or variables in PERL yet still having global variables the 'admin'
# can change after install-time.
# 
my $comm = Conf::group(); 

# jgw120717: Supposedly in normal operation an 'admin' user needs to be member of the Accreditation committee before s/he
#   can create any new committees (or working groups or whatever named voting group the end-users require.  I haven't yet 
#   proved that to be 100% true.  At any rate the Voting System seems to only allow voting to happen for 'accredited' committees. 
#
$dbh->do('INSERT INTO committees (comm, actstatus) VALUES ("$comm", 1), ("Accreditation", 1)');

# jgw120717: the 'admin' green is the first default green in the system, hence the value of greenid is set to = 2 here
#   at db-create / system install-time.
$dbh->do('INSERT INTO greenid (greenid) VALUES(2)');

$dbh->do('INSERT INTO propid (propid) VALUES(1)');

$dbh->do('INSERT INTO candid (candid) VALUES(1)');

$dbh->do('INSERT INTO roles (role) VALUES("Delegate"), ("Alternate"), ("Observer"), ("Admin"), ("Steering Committee"), ("Staff"), ("Co-Chair"), ("Member"), ("Advisor")');

# jgw120717: for the GPCA only ("California", "CA") is necessary.  Uncomment that line if that's all you want.
#$dbh->do('INSERT INTO states (state, abbr) VALUES("California", "CA")'); 
$dbh->do('INSERT INTO states (state, abbr) VALUES("Alabama", "AL"), ("Alaska", "AK"), ("Arizona", "AZ"), ("Arkansas", "AR"), ("California", "CA"), ("Colorado", "CO"), ("Connecticut", "CT"), ("Delaware", "DE"), ("District of Columbia", "DC"), ("Florida", "FL"), ("Georgia", "GA"), ("Hawaii", "HI"), ("Idaho", "ID"), ("Illinois", "IL"), ("Indiana", "IN"), ("Iowa", "IA"), ("Kansas", "KS"), ("Kentucky", "KY"), ("Louisiana", "LA"), ("Maine", "ME"), ("Maryland", "MD"), ("Massachusetts", "MA"), ("Michigan", "MI"), ("Minnesota", "MN"), ("Mississippi", "MS"), ("Missouri", "MO"), ("Montana", "MT"), ("Nebraska", "NE"), ("Nevada", "NV"), ("New Hampshire", "NH"), ("New Jersey", "NJ"), ("New Mexico", "NM"), ("New York", "NY"), ("North Carolina", "NC"), ("North Dakota", "ND"), ("Ohio", "OH"), ("Oklahoma", "OK"), ("Oregon", "OR"), ("Pennsylvania", "PA"), ("Rhode Island", "RI"), ("South Carolina", "SC"), ("South Dakota", "SD"), ("Tennessee", "TN"), ("Texas", "TX"), ("Utah", "UT"), ("Vermont", "VT"), ("Virginia", "VA"), ("Washington", "WA"), ("West Virginia", "WV"), ("Wisconsin", "WI"), ("Wyoming", "WY")');

# jw120616: I created & edited a clone of the 'states' table Insert statement into the states command, this
# Insert into counties command. Dridi's schema for the 'states' had no 'county abbreviation.'  The 'states' table is now setup 
# to hold all the abbreviations for all the counties of all the US states, provinces, territories.  
# How that might be useful in the future is still to be determined. 
#   There are all the 58 California Counties in the Insert statments here. 
# Also: THERE MAY NOT YET BE SUITABLE CODE IN THE VOTING SYSTEM TO USE THIS INFORMATION IN BALLOT RECORDING OR
# for the display of Voting Results.
#
# $dbh->do('INSERT INTO counties (county, abbr) VALUES("Alameda", "CA"), ("Alpine", "CA"), ("Amador", "CA"), ("Butte", "CA"), ("Calaveras", "CA"), ("Colusa", "CA"), ("Contra Costa", "CA"), ("Del Norte", "CA"), ("Diablo", "CA"), ("El Dorado", "CA"), ("Fresno", "CA"), ("Glenn", "CA"), ("Humbolt", "CA"), ("Imperial", "CA"), ("Inyo", "CA"), ("Kern", "CA"), ("Kings", "CA"), ("Lake", "CA"), ("Lassen", "CA"), ("Long Beach", "CA"), ("Los Angeles", "CA"), ("Madera", "CA"), ("Marin", "CA"), ("Mariposa", "CA"), ("Mendocino", "CA"), ("Mono", "CA"), ("Monterey", "CA"), ("Napa", "CA"), ("Nevada", "CA"), ("Orange", "CA"), ("Placer", "CA"), ("Riverside", "CA"), ("Sacramento", "CA"), ("San Benito", "CA"), ("San Bernadino", "CA"), ("San Diego", "CA"), ("San Francisco", "CA"), ("San Joaquin", "CA"), ("San Luis Obispo", "CA"), ("San Mateo", "CA"), ("Santa Barbara", "CA"), ("Santa Clara", "CA"), ("Santa Cruz", "CA"), ("Shasta", "CA"), ("Sierra", "CA"), ("Siskiyou", "CA"), ("Sonoma", "CA"), ("Stanislaus", "CA"), ("Sutter", "CA"), ("Tehama", "CA"), ("Trinity", "CA"), ("Tuolumne", "CA"), ("Tulare", "CA"), ("Ventura", "CA"), ("Yolo", "CA"), ("Yuba", "CA")');

$dbh->do('INSERT INTO phases (phase) VALUES("New"), ("Discussion"), ("Voting"), ("Certification"), ("Closed"), ("Cancelled")');

#First User - who is someone with admin priviledges for the Voting System program.
#  Enter the first and last names and email address for the Voting System program admin replacing 
#  the string In-All-Caps in the quotes below:
#
$dbh->do('INSERT INTO greens (gid, fname, lname, email) VALUES(1, "FNAME", "LNAME", "ADDY")');

#First User roles:
#  jgw120717: note that the 5th field in greens_roles, actstatus, is not touched here, but defaults to 1 = active role.
$dbh->do('INSERT INTO greens_roles (gid, commid, roleid, distid) VALUES(1, 1, 4, 0)');

#First User login - enter a login name and a password in the quotes below:

$dbh->do('INSERT INTO signons (gid, signon, pw) VALUES(1, "LOGIN", "PASSWORD")');

$dbh->disconnect;
exit();

