--
-- JW 12.06.17 for the GPCA
-- This SQL database (re)building files is based on the SQL in the  
-- create.pl script in Susan Dridi's 2004 VotePages IRV voting system
--
-- Edits are to 'timestamp(8)' now-equal-to 'timestamp'
--
-- Drop & Create Tables for VotePages database
-- Insert some VotePages system first-use record information in those tables
-- 

DROP TABLE IF EXISTS `greenid`;
DROP TABLE IF EXISTS `propid`;
DROP TABLE IF EXISTS `candid`;
DROP TABLE IF EXISTS `signons`;
DROP TABLE IF EXISTS `greens`;
DROP TABLE IF EXISTS `roles`;
DROP TABLE IF EXISTS `greens_roles`;
DROP TABLE IF EXISTS `greens_bios`;
DROP TABLE IF EXISTS `districts`;
DROP TABLE IF EXISTS `disttypes`;
DROP TABLE IF EXISTS `proposals`;
DROP TABLE IF EXISTS `phases`;
DROP TABLE IF EXISTS `votes`;
DROP TABLE IF EXISTS `irv`;
DROP TABLE IF EXISTS `irvvotes`;
DROP TABLE IF EXISTS `cands`;
DROP TABLE IF EXISTS `committees`;
DROP TABLE IF EXISTS `states`;
DROP TABLE IF EXISTS `counties`;


CREATE TABLE `greenid` (`greenid` int unsigned NOT NULL, PRIMARY KEY(`greenid`)) MAX_ROWS = 1 ENGINE=InnoDB ;

CREATE TABLE `propid` (`propid` int unsigned NOT NULL, PRIMARY KEY(`propid`)) MAX_ROWS = 1 ENGINE=InnoDB ;

CREATE TABLE `candid` (`candid` int unsigned NOT NULL, PRIMARY KEY(`candid`)) MAX_ROWS = 1 ENGINE=InnoDB ;

CREATE TABLE `signons` (`gid` int unsigned NOT NULL, `signon` varchar(20) NOT NULL, `pw` varchar(15) NOT NULL, `lastlogin` datetime, `sentreminder` tinyint unsigned, PRIMARY KEY(`gid`), UNIQUE KEY `signon` (`signon`), KEY `idx_sentreminder` (`sentreminder`)) ENGINE=InnoDB ;

CREATE TABLE `greens` (`gid` int unsigned NOT NULL, `fname` varchar(20) NOT NULL, `lname` varchar(20) NOT NULL, `add1` varchar(25), `add2` varchar(20), `city` varchar(25), `stateabbr` varchar(2), `zip` varchar(10), `email` varchar(80), `phd` varchar(20), `phe` varchar(20), `phc` varchar(20), `phf` varchar(20), `moddate` timestamp, PRIMARY KEY(`gid`), KEY `idx_lname` (`lname`), KEY `idx_fname` (`fname`), KEY `idx_moddate` (`moddate`)) ENGINE=InnoDB ;

CREATE TABLE `roles` (`roleid` smallint unsigned NOT NULL auto_increment, `role` varchar(25) NOT NULL, PRIMARY KEY(`roleid`)) ENGINE=InnoDB ;

CREATE TABLE `greens_roles` (`gid` int unsigned NOT NULL, `commid` smallint unsigned NOT NULL, `roleid` smallint unsigned NOT NULL, `distid` smallint unsigned NOT NULL, `actstatus` tinyint unsigned NOT NULL default '1', `moddate` timestamp NOT NULL, PRIMARY KEY(`gid`, `commid`, `roleid`, `distid`), KEY `idx_actstatus` (`actstatus`), KEY `idx_moddate` (`moddate`)) ENGINE=InnoDB ;

CREATE TABLE `greens_bios` (`gid` int unsigned NOT NULL, `bio` text NOT NULL, `pic` tinyint unsigned NOT NULL, PRIMARY KEY(`gid`), KEY `idx_pic` (`pic`)) ENGINE=InnoDB ;

CREATE TABLE `districts` (`distid` smallint unsigned NOT NULL auto_increment, `abbr` varchar(5), `district` varchar(30) NOT NULL, `disttypeid` tinyint unsigned NOT NULL, `voters` smallint unsigned NOT NULL, `maxvoters` smallint unsigned NOT NULL, `accreddate` date, `website` varchar(100), `actstatus` tinyint unsigned NOT NULL default '1', `moddate` timestamp NOT NULL, PRIMARY KEY(`distid`), UNIQUE KEY `district` (`district`), KEY `idx_disttypeid` (`disttypeid`), KEY `idx_voters` (`voters`), KEY `idx_maxvoters` (`maxvoters`), KEY `idx_accreddate` (`accreddate`), KEY `idx_actstatus` (`actstatus`), KEY `moddate` (`moddate`)) ENGINE=InnoDB ;

CREATE TABLE `disttypes` (`disttypeid` smallint unsigned NOT NULL auto_increment, `disttype` varchar(10) NOT NULL, PRIMARY KEY(`disttypeid`), KEY `idx_disttype` (`disttype`)) ENGINE=InnoDB ;

CREATE TABLE `committees` (`commid` smallint unsigned NOT NULL auto_increment, `comm` varchar(50) NOT NULL, webpage varchar(100), listserv varchar(100), actstatus tinyint unsigned NOT NULL default '1', `moddate` timestamp NOT NULL, PRIMARY KEY(`commid`), KEY `idx_actstatus` (`actstatus`), KEY `idx_moddate` (`moddate`)) ENGINE=InnoDB ;

CREATE TABLE `states` (`abbr` varchar(2) NOT NULL, `state` varchar(30) NOT NULL, PRIMARY KEY(`abbr`)) ENGINE=InnoDB ;

-- 
--  jw120616: added & edited clone of states table now = counties table. 
--  'abbr' field will now = 'state-code' abbr as in states table.
--  See "counties" inserts further below.
-- 

CREATE TABLE `counties` (`abbr` varchar(2) NOT NULL, `county` varchar(30) NOT NULL, PRIMARY KEY(`county`,`abbr`)) ENGINE=InnoDB ;

CREATE TABLE `proposals` (`pid` smallint unsigned NOT NULL, `title` varchar(100) NOT NULL, `gid` int unsigned NOT NULL, `presenter` varchar(100) NOT NULL, `background` text NOT NULL, `proposal` text NOT NULL, `resources` text, `reference` text, `discussbegin` date NOT NULL, `discussend` date NOT NULL, `votebegin` date NOT NULL, `voteend` date NOT NULL, `presens` double unsigned NOT NULL, `presensval` smallint unsigned, `consens` double unsigned NOT NULL, `consensval` smallint unsigned, `phaseid` tinyint unsigned NOT NULL, `distnum` smallint unsigned, `result` varchar(30), `moddate` timestamp NOT NULL, PRIMARY KEY(`pid`), KEY `idx_gid` (`gid`), KEY `idx_discussbegin` (`discussbegin`), KEY `idx_discussend` (`discussend`), KEY `idx_votebegin` (`votebegin`), KEY `idx_voteend` (`voteend`), KEY `idx_presens` (`presens`), KEY `idx_consens` (`consens`), KEY `idx_phaseid` (`phaseid`), KEY `idx_result` (`result`), KEY `idx_moddate` (`moddate`)) ENGINE=InnoDB ;

CREATE TABLE `phases` (`phaseid` tinyint unsigned NOT NULL auto_increment, `phase` varchar(25) NOT NULL, PRIMARY KEY(`phaseid`)) ENGINE=InnoDB ;

CREATE TABLE `votes` (`pid` smallint unsigned NOT NULL, `gid` int unsigned NOT NULL, `distid` smallint unsigned NOT NULL, `roleid` smallint unsigned NOT NULL, `vote` varchar(10) NOT NULL, `moddate` timestamp NOT NULL, PRIMARY KEY(`pid`, `gid`, `distid`), KEY `idx_roleid` (`roleid`), KEY `idx_vote` (`vote`), KEY `idx_moddate` (`moddate`)) ENGINE=InnoDB ;

CREATE TABLE `irv` (`pid` smallint unsigned NOT NULL, `title` varchar(100) NOT NULL, `gid` int unsigned NOT NULL, `numseats` tinyint unsigned NOT NULL, `background` text, `discussbegin` date NOT NULL, `discussend` date NOT NULL, `votebegin` date NOT NULL, `voteend` date NOT NULL, `presens` double unsigned NOT NULL, `presensval` smallint unsigned, `phaseid` tinyint unsigned NOT NULL, `distnum` smallint unsigned, `moddate` timestamp NOT NULL, PRIMARY KEY(`pid`), KEY `idx_gid` (`gid`), KEY `idx_numseats` (`numseats`), KEY `idx_discussbegin`  (`discussbegin`), KEY `idx_discussend` (`discussend`), KEY `idx_votebegin` (`votebegin`), KEY `idx_voteend` (`voteend`), KEY `idx_phaseid` (`phaseid`), KEY `idx_moddate` (`moddate`)) ENGINE=InnoDB ;

CREATE TABLE `cands` (`candid` smallint unsigned NOT NULL, `pid` smallint unsigned NOT NULL, `cand` varchar(50) NOT NULL, `info` text, `resources` text, `references` text, `candtype` tinyint unsigned NOT NULL, `equiv` tinyint unsigned, `result` varchar(30), `actstatus` tinyint unsigned NOT NULL default '1', `moddate` timestamp NOT NULL, PRIMARY KEY(`candid`,`pid`), KEY `idx_cand` (`cand`), KEY `idx_candtype` (`candtype`), KEY `idx_equiv` (`equiv`), KEY `idx_result` (`result`), KEY `idx_actstatus` (`actstatus`), KEY `idx_moddate` (`moddate`)) ENGINE=InnoDB ;

CREATE TABLE `irvvotes` (`pid` smallint unsigned NOT NULL, `voterkey` varchar(60) NOT NULL, `candid` smallint unsigned NOT NULL, `candrank` tinyint unsigned NOT NULL, PRIMARY KEY(`pid`,`voterkey`,`candid`), KEY `idx_candrank` (`candrank`)) ENGINE=InnoDB ;

-- Add values for supporting tables.

INSERT INTO `committees` (`comm`, `actstatus`) VALUES ("$comm", 1), ("Accreditation", 1) ;

INSERT INTO `greenid` (`greenid`) VALUES(2) ;

INSERT INTO `propid` (`propid`) VALUES(1) ;

INSERT INTO `candid` (`candid`) VALUES(1) ;

INSERT INTO `roles` (`role`) VALUES("Delegate"), ("Alternate"), ("Observer"), ("Admin"), ("Steering Committee"), ("Staff"), ("Co-Chair"), ("Member"), ("Advisor") ;

INSERT INTO `states` (`state`, `abbr`) VALUES("Alabama", "AL"), ("Alaska", "AK"), ("Arizona", "AZ"), ("Arkansas", "AR"), ("California", "CA"), ("Colorado", "CO"), ("Connecticut", "CT"), ("Delaware", "DE"), ("District of Columbia", "DC"), ("Florida", "FL"), ("Georgia", "GA"), ("Hawaii", "HI"), ("Idaho", "ID"), ("Illinois", "IL"), ("Indiana", "IN"), ("Iowa", "IA"), ("Kansas", "KS"), ("Kentucky", "KY"), ("Louisiana", "LA"), ("Maine", "ME"), ("Maryland", "MD"), ("Massachusetts", "MA"), ("Michigan", "MI"), ("Minnesota", "MN"), ("Mississippi", "MS"), ("Missouri", "MO"), ("Montana", "MT"), ("Nebraska", "NE"), ("Nevada", "NV"), ("New Hampshire", "NH"), ("New Jersey", "NJ"), ("New Mexico", "NM"), ("New York", "NY"), ("North Carolina", "NC"), ("North Dakota", "ND"), ("Ohio", "OH"), ("Oklahoma", "OK"), ("Oregon", "OR"), ("Pennsylvania", "PA"), ("Rhode Island", "RI"), ("South Carolina", "SC"), ("South Dakota", "SD"), ("Tennessee", "TN"), ("Texas", "TX"), ("Utah", "UT"), ("Vermont", "VT"), ("Virginia", "VA"), ("Washington", "WA"), ("West Virginia", "WV"), ("Wisconsin", "WI"), ("Wyoming", "WY") ;

-- jw120616: created & edited clone of Insert into states command, this
-- Insert into counties command. The scheme is setup to hold all counties of all US states.  
-- How that might be useful is to be determined. 
--   Currently there are only the 52 California Counties here. 
-- Also: THERE MAY NOT YET BE SUITABLE CODE IN THE VOTEPAGES SYSTEM TO USE THIS INFORMATION 
-- IN BALLOT RECORDING OR for the display of Voting Results.
-- 
INSERT INTO `counties` (`county`, `abbr`) VALUES("Alameda", "CA"), ("Alpine", "CA"), ("Amador", "CA"), ("Butte", "CA"), ("Calaveras", "CA"), ("Colusa", "CA"), ("Contra Costa", "CA"), ("Del Norte", "CA"), ("Diablo", "CA"), ("El Dorado", "CA"), ("Fresno", "CA"), ("Glenn", "CA"), ("Humbolt", "CA"), ("Imperial", "CA"), ("Inyo", "CA"), ("Kern", "CA"), ("Kings", "CA"), ("Lake", "CA"), ("Lassen", "CA"), ("Long Beach", "CA"), ("Los Angeles", "CA"), ("Madera", "CA"), ("Marin", "CA"), ("Mariposa", "CA"), ("Mendocino", "CA"), ("Mono", "CA"), ("Monterey", "CA"), ("Napa", "CA"), ("Nevada", "CA"), ("Orange", "CA"), ("Placer", "CA"), ("Riverside", "CA"), ("Sacramento", "CA"), ("San Benito", "CA"), ("San Bernadino", "CA"), ("San Diego", "CA"), ("San Francisco", "CA"), ("San Joaquin", "CA"), ("San Luis Obispo", "CA"), ("San Mateo", "CA"), ("Santa Barbara", "CA"), ("Santa Clara", "CA"), ("Santa Cruz", "CA"), ("Shasta", "CA"), ("Sierra", "CA"), ("Siskiyou", "CA"), ("Sonoma", "CA"), ("Stanislaus", "CA"), ("Sutter", "CA"), ("Tehama", "CA"), ("Trinity", "CA"), ("Tuolumne", "CA"), ("Tulare", "CA"), ("Ventura", "CA"), ("Yolo", "CA"), ("Yuba", "CA") ;

INSERT INTO `phases` (`phase`) VALUES("New"), ("Discussion"), ("Voting"), ("Certification"), ("Closed"), ("Cancelled") ;

-- First User - with admin priviledges for the program - enter the first and last names and email for the program admin in the quotes below:

INSERT INTO `greens` (`gid`, `fname`, `lname`, `email`) VALUES(1, "FNAME", "LNAME", "ADDY") ;

-- First User roles:

INSERT INTO `greens_roles` (`gid`, `commid`, `roleid`, `distid`) VALUES(1, 1, 4, 0) ;

-- First User login - enter a login name and a password in the quotes below:

INSERT INTO `signons` (`gid`, `signon`, `pw`) VALUES(1, "LOGIN", "PASSWORD") ;

